#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Apr  5 10:37:26 2021

@author: anton
"""

from SLiCAP import *

prj = initProject('Active antenna biasing')

def newCircuit(fileName):
    instr = instruction()
    makeNetlist(fileName + '.asc', fileName)
    instr.setCircuit(fileName + '.cir')
    htmlPage('Circuit data')
    head2html('Circuit diagram: ' + fileName + '.asc')
    img2html(fileName + '.svg', 600)
    netlist2html(fileName + '.cir')
    elementData2html(instr.circuit)
    instr.setSource('V1')
    instr.setDetector('V_out')
    instr.setSimType('symbolic')
    return(instr)

# Influence of R_B on dynamic response 
i1 = newCircuit('RB_Bandwidth')
i1.setGainType('gain')
i1.setDataType('laplace')
gainLaplace = i1.execute().laplace
htmlPage('Dynamic response')
head2html('Influence of $R_B$ on the small-signal transfer')
text2html('The source-to-load voltage transfer $A_v$ is found as:')
eqn2html('A_v', gainLaplace)
coeffs = coeffsTransfer(gainLaplace)
coeffsTransfer2html(coeffs)
head2html('Minimum value of $R_B$')
txt = """$R_B$ causes a high-pass transfer. The frequency of the pole should 
maximally equal the minimum frequency of interest $f_{min}$. From that we 
obtain the design equation:"""
text2html(txt)
eqn2html('f_min', 1/2/sp.pi/coeffs[2][1])
f_min = 10**4
R_B, C_A = sp.symbols('R_B, C_A')
CA = 6.3e-12
R_B_min_gain = sp.solve(1/2/sp.pi/coeffs[2][1]-f_min, R_B)[0].subs(C_A, CA)
text2html('From this we obtain:')
eqn2html('R_B_min', R_B_min_gain)

# Influence of C_c on dynamic response 
i1 = newCircuit('Cc_Bandwidth')
i1.setGainType('gain')
i1.setDataType('laplace')
gainLaplace = i1.execute().laplace
htmlPage('Dynamic response')
head2html('Influence of $C_c$ on the small-signal transfer')
text2html('The source-to-load voltage transfer $A_v$ is found as:')
eqn2html('A_v', gainLaplace)
coeffs = coeffsTransfer(gainLaplace)
coeffsTransfer2html(coeffs)
head2html('Minimum value of $C_c$')
txt = """$C_c$ causes a high-pass transfer. The frequency of the pole should 
maximally equal the minimum frequency of interest $f_{min}$. From that we 
obtain the design equation:"""
text2html(txt)
eqn2html('f_min', 1/2/sp.pi/coeffs[2][1])
f_min = 10**4
C_c, R_o, R_ell= sp.symbols('C_c, R_o, R_ell')
CA = 6.3e-12
C_c_min = sp.solve(1/2/sp.pi/coeffs[2][1]-f_min, C_c)[0].subs([(R_o, 50),(R_ell, 50)])
text2html('From this we obtain:')
eqn2html('C_c_min', C_c_min)

# Influence of R_B on the noise performance
i2 = newCircuit('RB_Noise')
i2.setGainType('vi')
i2.setDataType('noise')
noiseResult = i2.execute()
htmlPage('Symbolic noise analysis')
noise2html(noiseResult)
htmlPage('Numeric noise analysis')
i2.defPar('S_i', 0)
i2.defPar('S_v', 0)
i2.defPar('C_A', CA)
i2.defPar('R_o', 50)
i2.defPar('R_ell', 50)
i2.setSimType('numeric')
elementData2html(i2.circuit)
params2html(i2.circuit)
noiseResult = i2.execute()
noise2html(noiseResult)
txt = """$R_B$ increases the noise at low frequencies. A show stopper value is
obtained if its contribution at the lowest frequency of interest (10kHz) is 
just within specifications. Hence, we need to solve $R_B$ from:"""
text2html(txt)
lA    = 0.5 # antenna length
Sv10k = (200e-9*lA)**2 # noise voltage density at 10kHz in V^2/Hz
eqn2html(Sv10k, noiseResult.onoise.subs(ini.frequency, 10**4))
R_B_min_noise = sp.solve(noiseResult.inoise.subs(ini.frequency, 10**4)-Sv10k, R_B)
# We obtain two solutions, take the largest
R_B_min_noise = np.max(R_B_min_noise)
text2html("in this way we obtain:")
eqn2html('R_B_min', R_B_min_noise)

# Influence of RB on the output offset and voltage drive capability
i3 = newCircuit('RB_DCvar')
i3.setGainType('vi')
i3.setDataType('dcvar')
dcvarResult = i3.execute()
htmlPage('Symbolic DC variance analysis')
dcVar2html(dcvarResult)
txt = """$R_B$ converts the input bias current of the controller into an output
offset voltage. The total output offset voltage is the sum of the offset voltage
of the controller and the procuct of $R_B$ and the sum of the input bias current
and the input offset current of the controller."""
text2html(txt)
txt = """A show-stopper value of $R_B$ is found if the voltage drop across it
reduces the output voltage drive capability below the required value. An upper
limit for the total DC current that flows in the OpAmp (controller) terminals
can be found by considering the noise associated with this current. This current
noise can be converted into a source-referred voltage noise, similar as that of
$R_B$. A show-stopper value of the bias current can thus be found from:"""
text2html(txt)
k, T, q = sp.symbols('k, T, q')
eqn2html('2*q*I_b', 4*k*T/R_B)
text2html('From which we obtain:')
I_b_max = 2*k*T/R_B/q
I_b_max = I_b_max.subs([(k, 1.38e-23), (T, 300), (q, 1.6e-19), (R_B, R_B_min_noise)])
eqn2html('I_b_max', I_b_max)
txt = """Hence the maximum voltage drop across $R_B$ with $R_B=R_{B_{min_{noise}}}$
amounts:"""
text2html(txt)
eqn2html('V_dc_RB', I_b_max*R_B_min_noise, 'V')
txt = """Hence, with a low-bias OpAmp, the DC voltage drop across the bias 
resistor will not significantly reduce the load voltage drive capability."""
text2html(txt)